package com.x.mind.assemble.control.jaxrs.folder;

import com.google.gson.JsonElement;
import com.x.base.core.project.annotation.JaxrsDescribe;
import com.x.base.core.project.annotation.JaxrsMethodDescribe;
import com.x.base.core.project.annotation.JaxrsParameterDescribe;
import com.x.base.core.project.http.ActionResult;
import com.x.base.core.project.http.EffectivePerson;
import com.x.base.core.project.http.HttpMediaType;
import com.x.base.core.project.jaxrs.ResponseFactory;
import com.x.base.core.project.jaxrs.proxy.StandardJaxrsActionProxy;
import com.x.base.core.project.logger.Logger;
import com.x.base.core.project.logger.LoggerFactory;
import com.x.mind.assemble.control.ThisApplication;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.*;
import javax.ws.rs.container.AsyncResponse;
import javax.ws.rs.container.Suspended;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import java.util.List;

@Path("folder")
@JaxrsDescribe("脑图目录信息管理服务")
public class MindFolderInfoAction extends BaseAction {

	private StandardJaxrsActionProxy proxy = new StandardJaxrsActionProxy(ThisApplication.context());
	private Logger logger = LoggerFactory.getLogger(MindFolderInfoAction.class);

	@JaxrsMethodDescribe(value = "保存(创建或者更新)一个目录信息", action = ActionFolderSave.class)
	@POST
	@Path("save")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void saveFolder(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request, JsonElement jsonElement ) {
		EffectivePerson effectivePerson = this.effectivePerson(request);
		ActionResult<ActionFolderSave.Wo> result = new ActionResult<>();
		try {
			result =  ((ActionFolderSave)proxy.getProxy( ActionFolderSave.class ))
					.execute(request, effectivePerson, jsonElement);
		} catch (Exception e) {
			result = new ActionResult<>();
			logger.error(e, effectivePerson, request, null);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "将指定的脑图和文件夹移动到指定的文件夹内", action = ActionMoveToFolder.class)
	@PUT
	@Path("move/{folderId}")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void moveMindAndFolder(@Suspended final AsyncResponse asyncResponse, 
			@Context HttpServletRequest request, 
			@JaxrsParameterDescribe("文件夹ID") @PathParam("folderId") String folderId,
			JsonElement jsonElement ) {
		EffectivePerson effectivePerson = this.effectivePerson(request);
		ActionResult<ActionMoveToFolder.Wo> result = new ActionResult<>();
		try {
			result =  ((ActionMoveToFolder)proxy.getProxy( ActionMoveToFolder.class ))
					.execute(request, effectivePerson, folderId, jsonElement);
		} catch (Exception e) {
			result = new ActionResult<>();
			logger.error(e, effectivePerson, request, null);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "列出用户自己的个人脑图文件夹（所有文件夹，按上下级树型组织信息）", action = ActionListMyFolder.class)
	@GET
	@Path("tree/my")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void treeMyFolder(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request ) {
		EffectivePerson effectivePerson = this.effectivePerson(request);
		ActionResult<List<ActionListMyFolder.Wo>> result = new ActionResult<>();
		try {
			result =  ((ActionListMyFolder)proxy.getProxy( ActionListMyFolder.class ))
					.execute(request, effectivePerson);
		} catch (Exception e) {
			result = new ActionResult<>();
			logger.error(e, effectivePerson, request, null);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "根据ID删除一个目录，如果目录是非空目录，则无法删除）", action = ActionFolderDelete.class)
	@DELETE
	@Path("{id}")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void folderDelete(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("文件夹ID") @PathParam("id") String id ) {
		EffectivePerson effectivePerson = this.effectivePerson(request);
		ActionResult<ActionFolderDelete.Wo> result = new ActionResult<>();
		try {
			result =  ((ActionFolderDelete)proxy.getProxy( ActionFolderDelete.class ))
					.execute(request, effectivePerson,  id);
		} catch (Exception e) {
			result = new ActionResult<>();
			logger.error(e, effectivePerson, request, null);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "根据ID强制删除一个目录，包括下级目录以及文件", action = ActionFolderDeleteForce.class)
	@DELETE
	@Path("{id}/force")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void folderDeleteForce(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("文件夹ID") @PathParam("id") String id ) {
		EffectivePerson effectivePerson = this.effectivePerson(request);
		ActionResult<ActionFolderDeleteForce.Wo> result = new ActionResult<>();
		try {
			result =  ((ActionFolderDeleteForce)proxy.getProxy( ActionFolderDeleteForce.class ))
					.execute(request, effectivePerson,  id);
		} catch (Exception e) {
			result = new ActionResult<>();
			logger.error(e, effectivePerson, request, null);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
}