package com.x.organization.assemble.control.jaxrs.personcard;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.container.AsyncResponse;
import javax.ws.rs.container.Suspended;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;

import com.google.gson.JsonElement;
import com.x.base.core.project.annotation.JaxrsDescribe;
import com.x.base.core.project.annotation.JaxrsMethodDescribe;
import com.x.base.core.project.annotation.JaxrsParameterDescribe;
import com.x.base.core.project.http.ActionResult;
import com.x.base.core.project.http.EffectivePerson;
import com.x.base.core.project.http.HttpMediaType;
import com.x.base.core.project.jaxrs.ResponseFactory;
import com.x.base.core.project.jaxrs.StandardJaxrsAction;
import com.x.base.core.project.logger.Logger;
import com.x.base.core.project.logger.LoggerFactory;


@Path("personcard")
@JaxrsDescribe("个人通讯录操作")
public class PersonCardAction extends StandardJaxrsAction {

	private static Logger logger = LoggerFactory.getLogger(PersonCardAction.class);
	
	@JaxrsMethodDescribe(value = "获取个人名片.", action = ActionGet.class)
	@GET
	@Path("{flag}")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void get(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("名片标识") @PathParam("flag") String flag) {
		ActionResult<ActionGet.Wo> result = new ActionResult<>();
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionGet().execute(effectivePerson, flag);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, null);
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}

	@JaxrsMethodDescribe(value = "创建名片.", action = ActionCreate.class)
	@POST
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void create(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			JsonElement jsonElement) {
		ActionResult<ActionCreate.Wo> result = new ActionResult<>();
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionCreate().execute(effectivePerson, jsonElement);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, jsonElement);
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "更新个人名片.", action = ActionUpdate.class)
	@PUT
	@Path("{flag}")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void update(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("名片标识") @PathParam("flag") String flag, JsonElement jsonElement) {
		ActionResult<ActionUpdate.Wo> result = new ActionResult<>();
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionUpdate().execute(effectivePerson, flag, jsonElement);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, jsonElement);
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result)); 
	}

	@JaxrsMethodDescribe(value = "删除个人名片.", action = ActionDelete.class)
	@DELETE
	@Path("{flag}")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void delete(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("名片标识") @PathParam("flag") String flag) {
		ActionResult<ActionDelete.Wo> result = new ActionResult<>();
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionDelete().execute(effectivePerson, flag);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, null);
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "获取当前登录人的所有名片", action = ActionList.class)
	@GET
	@Path("mylist")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void mylist(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request) {
		ActionResult<List<ActionList.Wo>> result = new ActionResult<>();
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionList().MyExecute(effectivePerson);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, null);
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "获取当前登录人的名片.分页", action = ActionListPaging.class)
	@PUT
	@Path("listpaging/page/{page}/size/{size}")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8)
	@Consumes(MediaType.APPLICATION_JSON)
	public void ListPaging(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("页码") @PathParam("page") Integer page, @JaxrsParameterDescribe("每页几条") @PathParam("size") Integer size,
			@JaxrsParameterDescribe("姓名，手机号，电话，姓名拼音，分组，状态") JsonElement jsonElement) {
		ActionResult<List<ActionListPaging.Wo>> result = new ActionResult<>();
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionListPaging().Execute_Paging_like(effectivePerson, page, size, jsonElement);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, null);
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "按分组查询当前登录人的名片.分页", action = ActionListPagingWithGroup.class) 
	@PUT
	@Path("listpagingwithgroup/page/{page}/size/{size}")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8) 
	@Consumes(MediaType.APPLICATION_JSON)
	public void ListPagingWithGroupType(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("页码") @PathParam("page") Integer page, @JaxrsParameterDescribe("每页几条") @PathParam("size") Integer size,
			@JaxrsParameterDescribe("匹配关键字(分组)") JsonElement jsonElement) {
		ActionResult<List<ActionListPagingWithGroup.Wo>> result = new ActionResult<>(); 
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionListPagingWithGroup().Execute_Paging_groupType(effectivePerson, page, size, jsonElement);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, null);  
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "列出当前用户的所有分组", action = ActionListMyGroupTypes.class) 
	@GET
	@Path("listgrouptypes")
	@Produces(HttpMediaType.APPLICATION_JSON_UTF_8) 
	@Consumes(MediaType.APPLICATION_JSON)
	public void ListGroupTypes(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request) {
		ActionResult<String[]> result = new ActionResult<>(); 
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionListMyGroupTypes().MyExecute(effectivePerson);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, null);  
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "个人通讯录生成二维码", action = ActionCreateCode.class) 
	@GET
	@Path("createQR/{cardId}")
	@Consumes(MediaType.APPLICATION_JSON)
	public void code(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("名片id") @PathParam("cardId") String cardId) {
		ActionResult<ActionCreateCode.Wo> result = new ActionResult<>(); 
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionCreateCode().qrcode(effectivePerson, cardId);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, null);  
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "组织人员生成二维码", action = ActionPersonCode.class) 
	@GET
	@Path("createCode/{cardId}")
	@Consumes(MediaType.APPLICATION_JSON)
	public void personcode(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("名片id") @PathParam("cardId") String cardId) {
		ActionResult<ActionPersonCode.Wo> result = new ActionResult<>(); 
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionPersonCode().personcode(effectivePerson, cardId);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, null);  
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "导出组织通讯录vcf", action = ActionExportVcf.class) 
	@GET
	@Path("listVCf/{idList}")
	@Consumes(MediaType.APPLICATION_JSON)
	public void listVCf(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("名片id组") @PathParam("idList") String idList) {
		ActionResult<ActionExportVcf.Wo> result = new ActionResult<>(); 
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionExportVcf().exportVcf(effectivePerson, idList);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, null);  
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
	
	@JaxrsMethodDescribe(value = "导出个人通讯录vcf", action = ActionExportPersonalVcf.class) 
	@GET
	@Path("listPersonalVCf/{idList}")
	@Consumes(MediaType.APPLICATION_JSON)
	public void listPersonalVCf(@Suspended final AsyncResponse asyncResponse, @Context HttpServletRequest request,
			@JaxrsParameterDescribe("名片id组") @PathParam("idList") String idList) {
		ActionResult<ActionExportPersonalVcf.Wo> result = new ActionResult<>(); 
		EffectivePerson effectivePerson = this.effectivePerson(request);
		try {
			result = new ActionExportPersonalVcf().exportPersonalVcf(effectivePerson, idList);
		} catch (Exception e) {
			logger.error(e, effectivePerson, request, null);  
			result.error(e);
		}
		asyncResponse.resume(ResponseFactory.getEntityTagActionResultResponse(request, result));
	}
}